<h3>test</h3>


<% users = User.all%>
<table>
<tr>
	<th>name</th><th>C</th><th>I</th><th>S</th><th>filecount</th><th>issuecount</th><th>level</th>
</tr>
<% users.each do |user| %>



<tr>
	<td><%= user.lastname %><%= user.firstname  %></td>
	<td><%= calculate_collaboration_count(user) %></td>
	<td><%= calculate_influence_count(user) %></td>
	<td><%= calculate_skill_count(user) %></td>
 	<td><%= calculate_file(user) %></td>
 	<td><%= calculate_issue(user) %></td>
	<td><%= calculate_level(user) %></td>

</tr>
<% end %>
</table>

<hr/>

<hr/>
   